# Dołaczanie modułu pomiarowego ULPSM oraz SEN55 do Enviro Weather
# Na podstawie projektu 
#  "Pomiar prametrów powietrza w pomieszczeniach"
#  autorzy: Krzysztof KOSMALSKI Stanisław OBSZARSKI Jan ZAKRZEWSKI Mateusz ZENGEL
# ---------------------------------------------------------
import machine
from machine import ADC, Pin, UART, RTC, I2C
from pimoroni_i2c import PimoroniI2C
from picographics import PicoGraphics, DISPLAY_INKY_PACK
import enviro
from enviro.constants import *
from enviro import startup, sleep #i2c, 
from struct import unpack
import sys
sys.path.append('/enviro/boards/')

from weather import get_sensor_readings
import time
import utime
import random
import math

# Enviro startup
startup()

from time import sleep
from ADS1115 import *

import enviro.helpers as helpers
from pcf85063a import PCF85063A, MONDAY

# ---------------------------------------------------------
sleep_time = 2			# sleep time, minutes
sen_count_time = 3		# full measurement period counter
fan_count_time = round((10080/(sleep_time+1))/(sen_count_time))	# fan clean period counter

# ---------------------------------------------------------
# POBRANIE DATY I DANYCH Z POPRZEDNIEGO POMIARU ZAPISANYCH W PLIKU DATA.TXT
if helpers.file_exists("data.txt"):
  with open("data.txt", "r") as f:
    fs = f.readlines()[0].split(";")
    print("data.txt file: ", fs)
    file_flag = int(fs[0])
    if file_flag == 0:
        # Jednorazowe ustawienie data/time z pliku
        hour = int(fs[1])
        minute = int(fs[2])
        second = int(fs[3])
        day = int(fs[4])
        month = int(fs[5])
        year = int(fs[6])
        #file_flag = 2
    meas_counter = int(fs[7])
    fan_counter = int(fs[8])
    pm1p0 = float(fs[9])
    pm2p5 = float(fs[10])
    pm10p0 = float(fs[11])

else: # data.txt example 0;18;09;00;18;09;2024;3;840;1.0;2.5;10.0
    print("No data.txt file")
    file_flag = int(1)
    hour = int(15)
    minute = int(48)
    second = int(00)
    day = int(15)
    month = int(9)
    year = int(2024)
    meas_counter = int(sen_count_time)
    fan_counter = int(fan_count_time)
    pm1p0 = float(1.0)
    pm2p5 = float(5.0)
    pm10p0 = float(10.0)

# ---------------------------------------------------------
# Inicjalizacja i sprawdzanie polaczen
# BME280 0x77[119] 3,4MHz
# PCF85063A 0x51[81] 400kHz
# LTR-559ALS-01 0x23[35] 400kHz
# AS7341  0x39[57] 400kHz
# TCS34725 0x29[41] 400kHz
# ADS1115 0x48[72] 3,4MHz
# SEN555 0x69 [105] 100kHz
def check_connections():
    i2c1 = I2C(0, scl=Pin(5), sda=Pin(4), freq=100000) # GP4, GP5
    i2c_dev_list = i2c1.scan()
    if len(i2c_dev_list) == 0:
        print("No I2C devices found, terminating")
        sys.exit(1)
    print("LTR-559ALS-01 0x23, PCF85063A 0x51, BME280 0x77, ADS1115 0x48/0x49, SEN555 0x69")
    print("Detected devices at I2C1-addresses:")
    print(["0x{:02X}".format(x) for x in i2c_dev_list])
    print("Decimal address: ",i2c_dev_list)
    
    # Sprawdzanie ADS1115  0x48[72]
    if not 72 in i2c_dev_list:
        print("Failed to contact ADS1115")
        ads_flag = False
    else:
        ads1115_flag = True

    # Sprawdzanie SEN555 0x69 [105] 100kHz
    if not 105 in i2c_dev_list:
        print("Failed to contact SEN55")
        sen_flag = False
    else:
        sen_flag = True       

    return [sen_flag, ads1115_flag]

# ---------------------------------------------------------
# Check I2C devices
[sen_flag, ads1115_flag] = check_connections()

# ---------------------------------------------------------
# KONFIGURACJA RTC 
i2c = PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000)
rtc = PCF85063A(i2c)
i2c.writeto_mem(0x51, 0x00, b'\x00') # ensure rtc is running (this should be default?)
rtc.enable_timer_interrupt(False)

if file_flag == 0:
    # Jednorazowe ustawienie RTC data/time z pliku
    #rtc.datetime((year, month, day, hour, minute, second, , weekday))
    now = (year, month, day, hour, minute, second, 0)
    print("Neu RTC date/time: ", now)
    # Set PCF85063A RTC date/time
    rtc.datetime(now)
    file_flag = 2

t = rtc.datetime()
# BUG ERRNO 22, EINVAL, when date read from RTC is invalid for the pico's RTC.
RTC().datetime((t[0], t[1], t[2], t[6], t[3], t[4], t[5], 0)) # synch PR2040 rtc too
print("PCF85063A RTC date/time: ", t)

# ---------------------------------------------------------
# Wystartowanie pomiarów czujnika SEN55
if sen_flag >0: 
    # nawiazanie polaczenia po I2C oraz rozpoczecie pomiaru
    i2c = I2C(0, scl=Pin(5), sda=Pin(4), freq=100000)
    DEVICE_ADDR = 0x69	# SEN55
    # Check Device Status
    i2c.writeto(DEVICE_ADDR, bytes([0xD2, 0x06])) # Read Device Status
    time.sleep_ms(20)
    status = i2c.readfrom(DEVICE_ADDR, 4)
    SEN55_state = status[1] & 0x28 + status[3] & 0xF0
    print("SEN55 status", status, SEN55_state, " Meas counter:", meas_counter, " Fan counter:", fan_counter)
    if SEN55_state > 0:
        print("SEN55 status failed")
        sen_flag = 0
    else:
        # Czyszczenie wiatraczka SEN55 raz na tydzien
        print("SEN55 status OK")
        fan_counter -= 1
        if fan_counter == 0:
            # Start Fan Cleaning
            print("Start Fan Cleaning  Wait 10sec") 
            i2c.writeto(DEVICE_ADDR, bytes([0x56, 0x07])) # Read Device Status
            time.sleep(20)
            i2c.writeto(DEVICE_ADDR, bytes([0xD2, 0x06])) # Read Device Status
            time.sleep_ms(20)
            status = i2c.readfrom(DEVICE_ADDR, 4)
            SEN55_state = status[1] & 0x28
            print("SEN55 Fan status", status, SEN55_state)
            if SEN55_state > 0:
                print("SEN55 Fan status failed")
                sen_flag = 0
            else:
                print("SEN55 Fan status OK")
                fan_counter = fan_count_time

        # Start Measurement 
        if meas_counter >= sen_count_time: 
            # POMIAR PELNY  PM & RHT/Gas-Only
            i2c.writeto(DEVICE_ADDR, bytes([0x00, 0x21]))
        else:
            # POMIAR CZESCIOWY  RHT/Gas-Only
            i2c.writeto(DEVICE_ADDR, bytes([0x00, 0x37]))
        print("SEN55 measurent started")

# ---------------------------------------------------------
# CZYTANIE DANYCH Z CZUJNIKA CO ZA POMOCA ADS1115
# funkcja odpowiedzialna za odczyt napiecia przez ADC
def readChannel(channel):
    adc.setCompareChannels(channel)
    adc.startSingleMeasurement()
    while adc.isBusy():
        pass
    voltage = adc.getResult_V()
    return voltage

# konfiguracja ADS1115
ADS1115_ADDRESS = 0x48
i2c = I2C(0)
adc = ADS1115(ADS1115_ADDRESS, i2c=i2c)
adc.setVoltageRange_mV(ADS1115_RANGE_2048)
adc.setCompareChannels(ADS1115_COMP_0_GND)
adc.setConvRate(ADS1115_8_SPS) 
adc.setMeasureMode(ADS1115_SINGLE)

# odczyt napiecia
sample_count = 10
Vgas = 0
Vref = 0
for i in range(0, sample_count):
  Vgas += readChannel(ADS1115_COMP_0_GND)
  Vref += readChannel(ADS1115_COMP_1_GND)
Vgas /= sample_count
Vref /= sample_count
Vtemp = readChannel(ADS1115_COMP_2_GND)
adc.setVoltageRange_mV(ADS1115_RANGE_4096)
Vcc = readChannel(ADS1115_COMP_3_GND)

print("Vgas  ", Vgas)
print("Vref  ", Vref)
print("Vtemp ", Vtemp)
print("Vcc ", Vcc, " Vcc/2", Vcc/2)

# przeliczenie napiecia na wartosc poziomu CO obcecnego w powietrzu
Vgas0 = 0
M = 0.425/1000
C = (Vgas - Vref) / M
print("C ", C)
C = round(C)

# ---------------------------------------------------------
# ODCZYT Z ENVIRO WEATHER ZA POMOCA WBUDOWANEJ FUNKCJI IMPORTOWANEJ Z PLIKOW ENVIRO
pomiary = get_sensor_readings(15, False)

# ---------------------------------------------------------
# WYKONANIE POMIAROW Z SEN55
if sen_flag >0:
    # nawiazanie polaczenia po I2C oraz rozpoczecie pomiaru
    i2c = I2C(0, scl=Pin(5), sda=Pin(4), freq=100000)
    DEVICE_ADDR = 0x69	# SEN55

    # odczyt pomiarow 
    if meas_counter >= sen_count_time: # POMIAR PELNY  PM & RHT/Gas-Only
        print('SEN55 data reading all. Wait 60 sec')
        time.sleep(60) #20
        sen_sample_count = 10
        pm1p0 = 0
        pm2p5 = 0
        pm10p0 = 0
        for i in range(0, sen_sample_count):
            i2c.writeto(DEVICE_ADDR, bytes([0x02, 0x02]))
            time.sleep_ms(40)
            flag = i2c.readfrom(DEVICE_ADDR, 3)
            #print( "sen_sample_count", i, "flag", flag[1], "Meas counter", counter)
            if flag[1] == 0:
                time.sleep(1)
                print("Aditional 1s")
            i2c.writeto(DEVICE_ADDR, bytes([0x03, 0xC4]))  # Read Measured Values
            time.sleep_ms(20)
            msg = i2c.readfrom(DEVICE_ADDR, 24)
            pm1p0 += int(unpack(">h", msg[0:2])[0]) / 10
            pm2p5 += int(unpack(">h", msg[3:5])[0]) / 10
            pm10p0 += int(unpack(">h", msg[9:11])[0]) / 10
            print("cnt:", i, "flag", flag[1]," pm1p0", pm1p0, "pm2p5", pm2p5, "pm10p0", pm10p0)
            time.sleep(1)

        pm1p0 /= sen_sample_count
        pm2p5 /= sen_sample_count
        pm10p0 /= sen_sample_count
        voc = (msg[18] << 8 | msg[19]) / 10
        nox = (msg[21] << 8 | msg[22]) / 10
        tmp = (msg[15] << 8 | msg[16]) / 200
        hr = (msg[12] << 8 | msg[13]) / 100
        txt = "{:.2f}"
        print("avg:   ", "flag", flag[1]," pm1p0", txt.format(pm1p0), "pm2p5", txt.format(pm2p5), "pm10p0", txt.format(pm10p0), "tmp", txt.format(tmp), "hr", hr, " VOC", voc, " NOx", nox)
        meas_counter = 0
    else: # POMIAR CZESCIOWY  RHT/Gas-Only
        print('SEN55 data reading. Wait 60 sec')
        time.sleep(60)
        i2c.writeto(DEVICE_ADDR, bytes([0x03, 0xC4]))  # Read Measured Values
        time.sleep_ms(40)
        msg = i2c.readfrom(DEVICE_ADDR, 24)
        voc = (msg[18] << 8 | msg[19]) / 10
        nox = (msg[21] << 8 | msg[22]) / 10
        tmp = (msg[15] << 8 | msg[16]) / 200
        hr = (msg[12] << 8 | msg[13]) / 100
        txt = "{:.2f}"
        print("tmp", txt.format(tmp), "hr", hr, " VOC", voc, " NOx", nox)
        meas_counter += 1

    # Stop Measurement
    i2c.writeto(DEVICE_ADDR, bytes([0x01, 0x04]))  # Stop Measurement
    time.sleep_ms(20)
    print("Stop SEN55 Measurement")

# ---------------------------------------------------------
# WYSWIETLANIE DANYCH NA WYSWIETLACZU INKY_PACK
print("Printing data to display")
# konfiguracja wyswietlacza
display = PicoGraphics(DISPLAY_INKY_PACK)
WIDTH, HEIGHT = display.get_bounds()
display.set_update_speed(2)
display.set_font("bitmap6")
display.set_pen(100)
display.clear()
display.set_pen(0)

# mierzone parametry
display.text("TEMP: ", x1=0, y1=10, scale=2)
display.text("HUMI: ", x1=0, y1=30, scale=2)
display.text("PRES: ", x1=0, y1=50, scale=2)
display.text("LUMI: ", x1=0, y1=70, scale=2)
display.text("CO: ", x1=0, y1=90, scale=2)
display.text("VOC: ", x1=150, y1=10, scale=2)
display.text("NOx: ", x1=150, y1=30, scale=2)
display.text("PM1.0: ", x1=150, y1=50, scale=2)
display.text("PM2.5: ", x1=150, y1=70, scale=2)
display.text("PM10: ", x1=150, y1=90, scale=2)

# jednostki
display.text("°C", x1=120, y1=15, scale=1)
display.text("%H", x1=120, y1=35, scale=1)
display.text("hPa", x1=120, y1=55, scale=1)
display.text("LX", x1=120, y1=75, scale=1)
display.text("ppm", x1=120, y1=95, scale=1)
display.text("Idx", x1=260, y1=15, scale=1)
display.text("Idx", x1=260, y1=35, scale=1)
display.text("μg/m3", x1=260, y1=55, scale=1)
display.text("μg/m3", x1=260, y1=75, scale=1)
display.text("μg/m3", x1=260, y1=95, scale=1)

# wartosci
# sformatowac do dwoch miejsc po przecinku
display.text(str(pomiary.get("temperature")), x1=50, y1=10, scale=2)
display.text(str(pomiary.get("humidity")), x1=50, y1=30, scale=2)
display.text(str(pomiary.get("pressure")), x1=50, y1=50, scale=2)
display.text(str(pomiary.get("luminance")), x1=50, y1=70, scale=2)
display.text(str(C), x1=50, y1=90, scale=2)
display.text(str(voc), x1=205, y1=10, scale=2)
display.text(str(nox), x1=205, y1=30, scale=2)
display.text(str(txt.format(pm1p0)), x1=205, y1=50, scale=2)
display.text(str(txt.format(pm2p5)), x1=205, y1=70, scale=2)
display.text(str(txt.format(pm10p0)), x1=205, y1=90, scale=2)

# pobranie aktualnej daty oraz czasu, przekonwertowanie na stringi oraz odpowiednie sformatowanie danych
timedate = rtc.datetime() #(year, month, day, hour, minute, second, , weekday)
print("RTC date/time", timedate)
hour = timedate[3]
minute = timedate[4]
second = timedate[5]
if hour < 10:
    hour = "0" + str(hour)
else:
    hour = str(hour)
if minute < 10:
    minute = "0" + str(minute)
else:
    minute = str(minute)
if second < 10:
    second = "0" + str(second)
else:
    second = str(second)

day = timedate[2]
if day < 10:
    day = "0" + str(day)
else:
    day = str(day)
month = timedate[1]
if month < 10:
    month = "0" + str(month)
else:
    month = str(month)
year = str(timedate[0])

# wyswietlenie daty i godziny ostatniego pomiaru
if meas_counter == 0:
    display.text("Last meas: " + hour + ":" + minute + ":" + second + "* " + day + "." + month + "." + year, x1=0, y1=110, scale=2)
else:
    display.text("Last meas: " + hour + ":" + minute + ":" + second + "  " + day + "." + month + "." + year, x1=0, y1=110, scale=2)
display.update()

# ---------------------------------------------------------
# ZAKTUALIZOWANIE PLIKU DATA.TXT
print("Writing to the data.txt file")
with open("data.txt", "w") as f:
    f.write(str(file_flag) + ";" + hour + ";" + minute + ";" + second + ";" + day + ";" + month + ";" + year + ";" + str(meas_counter) + ";" + str(fan_counter)+ ";" + str(pm1p0) + ";" + str(pm2p5) + ";" + str(pm10p0))
    
# ---------------------------------------------------------
# PRZELACZENIE URZADZENIA W TRYB GLEBOKIEGO SNU NA OKRESLONY CZAS
print("Going to sleep")
enviro.sleep(sleep_time)
